<?php

namespace Drupal\orphans_media\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Trait ListFormTrait. List form trait.
 *
 * @package Drupal\orphans_media\Form
 */
trait ListFormTrait {

  /**
   * Build list form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildListForm(array &$form, FormStateInterface $form_state): array {
    // Get items per page.
    $itemsPerPage = intval($this->requestStack->getCurrentRequest()->query->get('items_per_page') ?? static::DEFAULT_ITEMS_PER_PAGE);

    // Get media entities.
    $total = $this->orphansMediaManager->getTotalUnusedMedias();
    $currentPage = $this->pagerManager->createPager($total, $itemsPerPage)
      ->getCurrentPage();
    $offset = $currentPage * $itemsPerPage;
    $requestedMediaBundles = $this->getRequestedMediaBundlesValues();
    $mediaEntities = $this->orphansMediaManager->getUnusedMedias($currentPage, $itemsPerPage, $offset, $requestedMediaBundles);

    // Build table.
    $rows = [];
    foreach ($mediaEntities as $media) {
      $rows[$media->id()] = [
        'title' => Markup::create('<strong>' . $media->label() . '</strong>'),
        'id' => Markup::create('<strong>' . $media->id() . '</strong>'),
        'type' => Markup::create('<strong>' . $media->bundle() . '</strong>'),
        'link' => Markup::create('<strong><a href="' . Url::fromRoute('entity.media.edit_form',
            [
              'media' => $media->id(),
            ])->toString() . '">' . $this->t('Edit') . '</a></strong>'),
      ];
    }

    // Build form.
    $form['filter']['items_per_page'] = [
      '#title' => $this->t('Items per page'),
      '#type' => 'select',
      '#options' => [25 => 25, 50 => 50, 75 => 75, 100 => 100],
      '#default_value' => $itemsPerPage,
    ];
    $form['filter']['media_bundle'] = [
      '#title' => $this->t('Media bundle'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#options' => $this->orphansMediaManager->getAvailableMediaBundles(),
      '#default_value' => $requestedMediaBundles,
    ];
    $form['filter']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
      '#submit' => [
        [$this, 'submitFilter'],
      ],
    ];

    $form['header'] = [
      '#type' => 'html_tag',
      '#tag' => 'h3',
      '#value' => $this->t('Total: @total', ['@total' => $total]),
    ];

    $tableHeader = [
      'title' => $this->t('Title'),
      'id' => $this->t('ID'),
      'type' => $this->t('Type'),
      'link' => $this->t('Link'),
    ];
    $form['table'] = [
      '#type' => 'tableselect',
      '#header' => $tableHeader,
      '#rows' => $rows,
      '#empty' => $this->t('No unused media found.'),
      '#options' => $rows,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete selected items'),
      '#validate' => [
        [$this, 'validateListForm'],
      ],
    ];

    // Pager.
    $form['pager'] = ['#type' => 'pager'];

    return $form;
  }

  /**
   * Validate list form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return void
   *   Nothing.
   */
  public function validateListForm(array &$form, FormStateInterface $form_state): void {
    $medias = array_filter($form_state->getValue('table'));
    if (0 === count($medias)) {
      $form_state->setError($form, $this->t('No items were selected to operate on.'));
    }
  }

  /**
   * Filter submit.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return void
   *   Nothing.
   */
  public function submitFilter(array &$form, FormStateInterface $form_state): void {
    $this->requestStack->getCurrentRequest()->query->set('items_per_page', $form_state->getValue('items_per_page'));
    $this->requestStack->getCurrentRequest()->query->set('media_bundle', $form_state->getValue('media_bundle'));
  }

  /**
   * Get default media bundles values.
   *
   * @return array
   *   The default media bundles values.
   */
  public function getRequestedMediaBundlesValues(): array {
    $parameters = $this->requestStack->getCurrentRequest()->query->all();
    return $parameters['media_bundle'] ?? [];
  }

}
